# COPYRIGHT 2009 Hannes Isaak Reuter
# All rights reserved
# Contact: gisxperts@web.de
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#
#---------------------------------------------------------------------------
#/* aml for delineation of landforms according to Iwahashi and Pike (2006)
#program for automated classification of topography
# /* by Junko IWAHASHI, Geographical Survey Institute of Japan
#/*    October, 2005
#/*      dem: DEM image (Arc/INFO GRID format); NODATA area must be defined
#/*           for waterbodies
#/*      laplacian: kernel file (ASCII text) for 3*3 laplacian filter
#/*      classnumber: ASCII number (8, 12 or 16)
#/* &r act <inputdem> <outputdem> <kernelfile> {classnumber}
#/* coded in python - Hannes Isaak Reuter
# F:\jrc_d_drive\geomorphometryTB\test\d25m2 F:\jrc_d_drive\geomorphometryTB\test\d25m2_act16 16 # # # # # # # # # # #
# D:\geomorphometryTB\test\d25m2 D:\geomorphometryTB\test\d25m2_act16 16 # # # # # # # # # # #
# D:\geomorphometryTB\test\d25m2" "D:\geomorphometryTB\test\d25m2_8" "8" "#" "#" "#" "#" "#" "#" "#" "#"
# F:\jrc_d_drive\geomorphometryTB\test\ire F:\jrc_d_drive\geomorphometryTB\test\ire_l1act16 "16" "#" "#" "#" "#" "#" "#" "#" "#" "#" "#" "#"
# D:\projects\largetest\s30l1 D:\projects\largetest\s30l1_act163 16 # # # # # # # # # # #
# F:\jrc_d_drive\geomorphometryTB\test\d25 F:\jrc_d_drive\geomorphometryTB\test\d25_python16 "16" "#" "#" "#" "#" "#" "#" "#" "#" "#" "#" "#"
# F:\jrc_d_drive\geomorphometryTB\test\bire F:\jrc_d_drive\geomorphometryTB\test\bire_act16  "16" "#" "#" "#" "#" "#" "#" "#" "#" "#" "#" "#"
def kernellaplacian():
    gp.AddMessage("Write laplacian kernel file")
    fd,aml = tempfile.mkstemp(suffix='.txt',dir=workspace)
    tmp = os.fdopen(fd, 'w+t')
    # print aml
    cmd="3 3" ;        tmp.write(cmd+'\n')
    cmd="0 -1 0" ;        tmp.write(cmd+'\n')
    cmd="-1 4 -1" ;        tmp.write(cmd+'\n')
    cmd="0 -1 0" ;        tmp.write(cmd+'\n')
    tmp.close()
    return(aml)

# Import system modules
import sys, string, os,  tempfile, traceback,textwrap
# Create the Geoprocessor object
version = sys.version[0:4]
if version == '2.1.':
    import win32com.client,tp21
    gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")
elif version == '2.4.':
    import arcgisscripting
    import tp24 as tp
    gp = arcgisscripting.create()
elif version == '2.5.':
    import arcgisscripting
    import tp25 as tp
    gp = arcgisscripting.create()
elif version == '2.6.':
    import arcgisscripting
    import tp26 as tp
    gp = arcgisscripting.create()
else:
    raise Exception, 'Version not yet supported'


ARCGIS = ""
ARCINFO = ""
# Script arguments...
Expression = sys.argv[1]
outputDEM = sys.argv[2]
classnumber = sys.argv[3]
if not classnumber in ('8','12','16'):
    tp.gtberror("Numbers not specified or allowed -only 8-12-16",'',gp)
if classnumber == '#':
    hlparam = "16"
wpitpeak = sys.argv[4]
if wpitpeak == '#' :
    wpitpeak = 10
wconvex = sys.argv[5]
if wconvex == '#' :
    wconvex = 10
si = sys.argv[6]
co = sys.argv[7]
pp = sys.argv[8]
si1 = sys.argv[9]
co1 = sys.argv[10]
pp1 = sys.argv[11]
si2 = sys.argv[12]
co2 = sys.argv[13]
pp2 = sys.argv[14]
# DEBUG = True
if len(sys.argv) >= 16:
    DEBUG = sys.argv[15]
    if DEBUG == '#':
        DEBUG = 0
else:
    DEBUG = 0
# setup
ARCGIS,ARCINFO,scratchWS = tp.geomorphometrysetup(gp,sys,os,DEBUG)
# create variables for workspace
dem,workspace = tp.getnameandworkspace_metric(Expression,os,gp,string,DEBUG)
gp.workspace = workspace
# test for out
outdem,outworkspace = tp.checkoutput(outputDEM,os,gp)
# Local variables...
dem_fldir = workspace+dem + "fldir"
simg  = workspace + dem + "simg"
pitpeak  = workspace + dem + "pitpeak"
convex  = workspace + dem + "convex"


try:
    # clean temp files
    tp.cleanttempfile4(gp,scratchWS,workspace,os)
    # set extent
    if (wpitpeak >= wconvex):
        winext = wpitpeak
    else:
        winext = wconvex
    gp.extent = tp.getextentforxcells(dem,0,gp)
    # get filter file
    laplacian = kernellaplacian()
    #laplacian = "Irregular "+laplacian+" CELL"
    laplacian = "Weight "+string.replace(laplacian,'\\','\\\\')+" CELL"
    #/* prepare all input data for classification process.
    cellnumber = tp.getcellnumber(dem,gp)
    if ( cellnumber > 120000000 ):
            tp.actlargeareapreprocessing(dem,simg,pitpeak,convex,laplacian,wconvex,wpitpeak,gp,os,scratchWS)
            # we need to mosaic it fa, fb and fc tiles already in ACTpreprocess
    else:
            tp.actsmallareapreprocessing(dem,simg,pitpeak,convex,laplacian,wconvex,wpitpeak,gp,os,scratchWS)
#/* start classification
    gp.extent = tp.getextentforxcells(dem,winext,gp)
    if version == '2.6.':
        tp.actclassificationnumpy(dem,outputDEM,simg,pitpeak,convex,classnumber,si,co,pp,si1,co1,pp1,si2,co2,pp2,ARCINFO,workspace,tempfile,gp,os,scratchWS,textwrap)
    else:
        tp.actclassification(dem,outputDEM,simg,pitpeak,convex,classnumber,si,co,pp,si1,co1,pp1,si2,co2,pp2,ARCINFO,workspace,tempfile,gp,os,scratchWS,textwrap)
    # clean temp files
    if not DEBUG:
        tp.cleanttempfile4(gp,scratchWS,workspace,os)

    gp.CheckInExtension("spatial")
    tp.gtbmsgid(25,'',gp)
    tp.gtbmsg("Finished for iwahashi.py for input: "+dem+' into output: ',outputDEM,gp)
    

except:
##/* ######################################################
##/* routine bailout start
##/* ######################################################
  tp.errorcall(sys,gp,traceback)
