# COPYRIGHT 2009 Hannes Isaak Reuter
# All rights reserved
# Contact: gisxperts@web.de
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#
##/* NAME: landform.aml ( currentversion pennock96.aml)
##/* WHAT: landform classification analysis
##/* INPUT: DEM, running topo.aml as prequisite,
##/*             default parameter for 10m x 10m DEM
##/* DEVELOPER: hannes isaak reuter, zalf muencheberg, germany
##/* CONTACT:  hreuter@zalf.de / gisxperts@web.de
##/* CHANGES: HIR 1999-2002 phd thesis "spatial crop and soil landscape processes
##/* 			 under special consideration of relief information in a loess landscape"
##/* 			 Uni hannover, series horizonte, probably bd.16
##/* 	    HIR 23/06/2003 cleaning for arcsripts upload....
##/* 	    HIR 24/05/2004 Fixed error in 8 landform classification routine - thanx yang
##/* COMMENTS:
##/* 	does landform classification after original paper by pennock 1994
##/* 	and further literature sources
##/* 	does extended landform classification for planar landforms,
##/*	preserves shape of each landform, not only SH,BS,FS, and Level

##/* 	method is the 8 or 11 relief classes , default is set to 11 classes
##/* 	Citation:
##/* 	1) Reuter,H.I. 2002
##/* 	Computing Landforms using a Digital Elevation Model inside ArcINFO
##/* 	2) used and described in paper:
##/*      MOSAIC:Crop yield observation - can landform stratification improve
##/* 	our understanding of crop yield variability.
##/*	4.European Conference on Precision Agriculture
##/* 	Berlin, 2003, ISBN 9076998213, page 579-584,
## F:\jrc_d_drive\geomorphometryTB\test\d25 F:\jrc_d_drive\geomorphometryTB\test\d25pen11 F:\jrc_d_drive\geomorphometryTB\test\d25pen11r 11 0.01 0.01 3.0 500
import sys, string, os, tempfile, traceback
# Create the Geoprocessor object
version = sys.version[0:4]
if version == '2.1.':
    import win32com.client,tp21
    gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")
elif version == '2.4.':
    import arcgisscripting
    import tp24 as tp
    gp = arcgisscripting.create()
elif version == '2.5.':
    import arcgisscripting
    import tp25 as tp
    gp = arcgisscripting.create()
elif version == '2.6.':
    import arcgisscripting
    import tp26 as tp
    gp = arcgisscripting.create()
else:
    raise Exception, 'Version not yet supported'

ARCGIS = ""
ARCINFO = ""
scratchWS = ""
# Script arguments...
Expression = sys.argv[1]
output = sys.argv[2]
output1 = sys.argv[3]
method = sys.argv[4]
profile = sys.argv[5]
planform = sys.argv[6]
slope = sys.argv[7]
warea = sys.argv[8]
# setup
if len(sys.argv) >= 10:
    DEBUG = sys.argv[9]
    if DEBUG == '#':
        DEBUG = 0
else:
    DEBUG = 0
if method == '#':
    method = 11
if profile == '#':
    profile = 0.01
if planform == '#' and  method == 8 :
    planform = 0.0
elif planform == '#' and  method == 11 :
    planform = 0.01
if warea == '#':
    warea = 500
if slope == '#':
    slope = 3.0
# how to handle nodata
InNoDataOption = "DATA"
# setup
ARCGIS,ARCINFO,scratchWS = tp.geomorphometrysetup(gp,sys,os,DEBUG)
dem,workspace = tp.getnameandworkspace_nl5metric(Expression,os,gp,string,DEBUG)
gp.workspace = workspace
outdem,outworkspace = tp.checkoutput(output,os,gp)
outdem,outworkspace = tp.checkoutput(output1,os,gp)
# set names
dem_pennock = output
dem_pennock_reclass = output1
dem_slp= dem +'slp'
dem_plan= dem +'plan'
dem_prof= dem +'prof'
dem_flacc = dem +'flacc'
dem_curv =dem +'curv'
# local variables

try:
    # clean temp files
    tp.cleanttempfile4(gp,scratchWS,workspace,os)
    # test if neccessary input files are there
    if not gp.Exists(dem_flacc):
            tp.justflowacc(dem,gp,scratchWS)
            
    if not gp.exists(dem_prof) or not gp.exists(dem_plan):
        tp.gtbmsgid(60,dem_curv,gp)
        inzfactor = "1.0"
        gp.Curvature_sa(dem, dem_curv, inzfactor, dem_prof, dem_plan)
    else:
        tp.gtbmsgid(61,dem_curv,gp)

    if not gp.exists(dem_slp):
        tp.gtbmsgid(62,dem_slp,gp)
        gp.Slope_sa(dem, dem_slp, "DEGREE", "1")
    else:
        tp.gtbmsgid(63,dem_slp,gp)
# classsify
    if version == '2.6.':
        tp.pennockclassnumpy(method,dem,dem_prof,dem_plan, dem_slp,dem_flacc,output,output1,slope, profile, planform, warea, ARCINFO,workspace,gp,os,tempfile)
        import arcpy as apy
        from arcpy import env
        from arcpy.sa import *
        if method == 8:
            outReclassify = apy.sa.Reclassify(output, 'Value', apy.sa.RemapRange([[1,2, 1],[3,4, 2],[5,6, 3],[7,8, 4]]), "NODATA")
            outReclassify.save(output1)
        else:
            outReclassify = apy.sa.Reclassify(output, 'Value', apy.sa.RemapRange([[1,3, 1],[4,6, 2],[7,9, 3],[10,11, 4]]), "NODATA")
            outReclassify.save(output1)
    else:
        tp.pennockclass(method,dem,dem_prof,dem_plan, dem_slp,dem_flacc,output,output1,slope, profile, planform, warea, ARCINFO,workspace,gp,os,tempfile)
# done
    gp.CheckInExtension("spatial")

    if not DEBUG :
        tp.cleanttempfile4(gp,scratchWS,workspace,os)

    tp.gtbmsgid(25,'',gp)
    tp.gtbmsg("Finished ok for pennock.py for ",output,gp)
    tp.gtbmsg("Finished ok for pennock.py for ",output1,gp)


except:
##/* ######################################################
##/* routine bailout start
##/* ######################################################
  tp.errorcall(sys,gp,traceback)