# COPYRIGHT 2009 Hannes Isaak Reuter
# All rights reserved
# Contact: gisxperts@web.de
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#
##/* classify several grid for
##/* does landform classification after canadian soil landscape models
##/*
##/* implements an filtering algorithm to reduce the number of
##/* misclassified relief units
##/* written by hannnes isaak reuter
##/* contact by hannes.reuter(at)jrc.it or gisxperts@web.de
##/* zalf muencheberg
##/* DFG project mosaik 1999 - 2002
##/* cite: H.I.Reuter, 2003, Simple Landform classification in AML based on Canadian Soil Landscape Models,
##/*
##/* method can classify 4 (a), 8 (b) or generalized 8 reliefunits (c)
##/* method reference: Macmillan,R.A. and Pettapiece,W.W., 1997. Soil Landscape Models: Automated landscape characterization and generation of landscape models
##/* method reference: Technical Bulletin 1997-1E, Agriculture and Agri-Food Canada, Research Branch, Lethbridge,AB. 75pp
##/* the variable pennock is the area threshold, default is set to 5
##/* improvevments - need to define an iteration threshold to
##/* and implement it, especially for large data sets
##/* default of slope is 2
##/* default of area threshold is 5 cells
##/* expects parameters SLOPE and PCTG1 which will be computed by TOPO.AML, most simple approach..
##/* otherwise you can create them on your own if you have a dem (%dem%) and a watershed grid (%dem%wshd):
##/* %dem%slp = slope ( %dem% )
##/* zonalamax for watershed
##/* tmp12345 = zonalmax ( %dem%wshd, %dem% )
##/* zonalmin for watershed
##/* tmp12346 = zonalmin ( %dem%wshd, %dem% )
##/* pctg1
##/* %file%pctg1 = (( %file% - tmp12346 ) / ( tmp12345 - tmp12346 ) ) * 100
## C:\WorkSpace\test\d25m2 C:\WorkSpace\test\a25msimplea a 2 20 40 70
import sys, string, os, tempfile,  traceback
# Create the Geoprocessor object
version = sys.version[0:4]
if version == '2.1.':
    import win32com.client,tp21
    gp = win32com.client.Dispatch("esriGeoprocessing.GpDispatch.1")
elif version == '2.4.':
    import arcgisscripting
    import tp24 as tp
    gp = arcgisscripting.create()
elif version == '2.5.':
    import arcgisscripting
    import tp25 as tp
    gp = arcgisscripting.create()
elif version == '2.6.':
    import arcgisscripting
    import tp26 as tp
    gp = arcgisscripting.create()
else:
    raise Exception, 'Version not yet supported'

ARCGIS = ""
ARCINFO = ""
scratchWS = ""
# Script arguments...
Expression = sys.argv[1]
dem_out = sys.argv[2]
method = sys.argv[3]
# check for method
if not method in ('a','b','c'):
    tp.gtberror("Method not specified or allowed -only a-b-c",'',gp)
slope = sys.argv[4]
if slope == '#':
  slope = 2
thr1 = sys.argv[5]
if thr1 == '#' :
  thr1 = 20.0
thr2 = sys.argv[6]
if thr2 == '#' :
  thr2 = 40.0
thr3 = sys.argv[7]
if thr3 == '#' :
  thr3 = 70.0
if len(sys.argv) >= 9:
    DEBUG = sys.argv[8]
    if DEBUG == '#':
        DEBUG = 0
else:
    DEBUG = 0
# setup
ARCGIS,ARCINFO,scratchWS = tp.geomorphometrysetup(gp,sys,os,DEBUG)
dem,workspace = tp.getnameandworkspace_metric(Expression,os,gp,string,DEBUG)
gp.workspace = workspace
outdem,outworkspace = tp.checkoutput(dem_out,os,gp)
# how to handle nodata
InNoDataOption = "DATA"
# local variables
dem_slope= dem +'slp'
dem_pctg1= dem +'pctg1'
try:
# clean temp files
    tp.cleanttempfile4(gp,scratchWS,workspace,os)
    tp.getslope (dem, dem_slope, "DEGREE", "1",gp)
    if not gp.Exists(dem_pctg1):
        tp.gtberrorid(12,dem_pctg1,gp)
    else:
        tp.gtbmsgid(115,dem_pctg1,gp)
    # define windows smaller than the original values
    gp.extent = tp.getextentminus1(dem,gp)
    tp.gtbmsg(" Process DEM "+dem+ " with Parameters SLOPE "+ str(slope)+" and thresholds 1 and 2 and 3 : "+str(thr1)+" "+str(thr2)+" "+str(thr3),'',gp)
    tp.gtbmsg("Classification will use method: ",method,gp)
    # now classify
    if version == '2.6.':
        tp.simpleLFnumpy(dem_pctg1,dem_slope,dem_out,method,slope,thr1,thr2,thr3,gp,ARCINFO,workspace,tempfile,os)
    else:
        tp.simpleLF(dem_pctg1,dem_slope,dem_out,method,slope,thr1,thr2,thr3,gp,ARCINFO,workspace,tempfile,os)

    tp.gtbmsgid(25,'',gp)
    tp.gtbmsg("Finished ok for simplelfabc.py for "+dem_out+' and method ',method,gp)
    if not DEBUG :
        tp.cleanttempfile4(gp,scratchWS,workspace,os)
except:
##/* ######################################################
##/* routine bailout start
##/* ######################################################
  tp.errorcall(sys,gp,traceback)